/*
 * burton rosenberg
 * july 2004
 * version 1
 */

import java.io.* ;

class Playfair {

   class RowCol { int row ; int col ; }
   class CharPair { char c_one; char c_two ; }

   char [] pfTable ;

   Playfair( String theTable ) throws Exception {
       if ( theTable.length()!=25 ) throw new Exception() ;
       PlayfairInputStream sis = new PlayfairInputStream(theTable) ;
       pfTable = new char[25] ;
       for ( int i=0; i<25; i++ ) {
          pfTable[i] = sis.getNextChar() ;
       }
   }
      
   RowCol lookupChar( char c ) {
       for ( int i=0; i<pfTable.length; i++ ) {
          if ( pfTable[i]==c ) {
             RowCol rc = new RowCol() ;
             rc.row = i/5 ;
             rc.col = i%5 ;
             return rc ;
          }
       }
       return null ;
   }

   char lookupRowCol( RowCol rc ) { 
       return pfTable[ 5*rc.row+rc.col ] ;
   }

   CharPair encodeCharPair( CharPair charPair ) {
       return encodeCharPair( charPair, +1 ) ;
   }

   CharPair decodeCharPair( CharPair charPair ) {
       return encodeCharPair( charPair, +4 ) ;
   }

   CharPair cp = new CharPair() ;

   CharPair encodeCharPair( CharPair charPair, int dir ) {

       // dir = +1 encode, +4 decode 

       RowCol rc0 = lookupChar( charPair.c_one ) ;
       RowCol rc1 = lookupChar( charPair.c_two ) ;
       if ( rc0==null || rc1==null ) return null ;

       if ( rc0.row == rc1.row ) {
          // row arrangement
          rc0.col = (rc0.col+dir)%5 ;
          rc1.col = (rc1.col+dir)%5 ;
       }
       else if ( rc0.col == rc1.col ) {
          // col arrangement
          rc0.row = (rc0.row+dir)%5 ;
          rc1.row = (rc1.row+dir)%5 ;
       }
       else {
          // square arrangement 
          int t = rc0.col ;
          rc0.col = rc1.col ;
          rc1.col = t ;
       }

       cp.c_one =lookupRowCol( rc0 ) ;
       cp.c_two =lookupRowCol( rc1 ) ;
       return cp ;
   }

  void encode( PlayfairInputStream is, PlayfairOutputStream os ) {
      char c ;
      while ( (c=is.getNextChar()) != (char) 0 ) {
         cp.c_one = c ;
         cp.c_two = is.getNextChar() ;
         cp = encodeCharPair( cp ) ;
         os.putChar(cp.c_one) ;
         os.putChar(cp.c_two) ;
      }
      os.close() ;

  }

  void decode( PlayfairInputStream is, PlayfairOutputStream os ) {
      char c ;
      while ( (c=is.getNextChar()) != (char) 0 ) {
         cp.c_one = c ;
         cp.c_two = is.getNextChar() ;
         cp = decodeCharPair( cp ) ;
         os.putChar(cp.c_one) ;
         os.putChar(cp.c_two) ;
      }
      os.close() ;

  }

  public static void main(String [] args) {

      boolean isDecode = false ;
      boolean isEncode = false ;

      if ( args.length!=2 ) {
        printUsage() ;
      }

      if ( "-d".equals(args[0]) ) isDecode = true ;
      if ( "-e".equals(args[0]) ) isEncode = true ;

      if ( !isDecode && !isEncode ) {
         printUsage() ;
      }
     
      Playfair playfair = null ;
      try {
         playfair = new Playfair( args[1] ) ;
      } catch ( Exception ex ) { 
         System.out.println("exception: could not create playfair table.") ;
         System.out.println("   check key length, must be 25 characters" ) ;
         System.exit(1) ;
      }
      PlayfairInputStream pis = new PlayfairInputStream() ;
      PlayfairOutputStream pos = new PlayfairOutputStream() ;
      if ( isDecode ) playfair.decode(pis, pos) ;
      if ( isEncode ) playfair.encode(pis, pos) ;

  }
 
  static void printUsage() {
      System.out.println("Usage: [-e|-d] key") ;
      System.exit(0) ;
  }
}

